<?php
/*
Plugin Name: GPN DATA International Payment Gateway for WooCommerce
Plugin URI: https://gpndata.com
Description: GPN DATA International Payment Gateway for secure transaction processing in WooCommerce.
Version: 1.0.13
Author: GPN Data Dev Team
Author URI: https://gpndata.com
License: Proprietary
 */

/**
 * txpmnts
 * Provides an interface to GPN DATA International iFrame
 * @class      WC_txpmnts
 * @package    WooCommerce
 * @category   Payment Gateways
 * @author     GPN Data Dev Team
 */


if(!class_exists('WC_Payment_Gateway')){
	require_once(dirname(__FILE__).'/../woocommerce/woocommerce.php');
}

class WC_txpmnts extends WC_Payment_Gateway{

	public $_db;
	public $gpndata_order;
	protected $languages = null;
	protected $selected_language = null;
	public $_API = null;
    private $_default_iframe_style =
        '#outer {
            height: 58em;
            text-align: center;
        }
        #inner {
            margin: 0 auto;
            width: 100%;
            height: 100%;
        }';
    public $supports = array('products', 'refunds');
	public function __construct(){

		$this->id = 'txpmnts';
		$this->has_fields   = false;
		$this->method_title = __('TxPayments', 'woocommerce');

		// Load the form fields.
		$this->init_form_fields();

		// Load the settings.
		$this->init_settings();

		// Load the default language
		$this->selected_language = 'en-gb';

		// Define user set variables
        $this->title                  = $this->get_option('title');
		$this->gateway_url            = $this->get_option('gateway_url');
		$this->iframe_url             = $this->get_option('iframe_url');
		$this->pretty_name            = $this->get_option('pretty_name');
        $this->iframe_language        = $this->get_option('iframe_language');
        $this->iframe_order_description = $this->get_option('iframe_order_description');
		$this->apiuser                = $this->get_option('apiuser');
		$this->apipassword            = $this->get_option('apipassword');
		$this->apikey                 = $this->get_option('apikey');
		//$this->contacturl             = $this->get_option('contacturl');
		$this->logo            		  = $this->get_option('logo');
		$this->merchantname           = $this->get_option('merchantname');
		$this->description            = $this->get_option('description');
		$this->email                  = $this->get_option('email');
		$this->send_shipping          = $this->get_option('send_shipping');
		$this->address_override       = $this->get_option('address_override');
		$this->debug                  = $this->get_option('debug');
		$this->form_submission_method = $this->get_option('form_submission_method') === 'yes' ? true : false;
		$this->page_style             = $this->get_option('page_style') ? $this->get_option('page_style') : '';
        $this->languages              = array('en-gb' => include (dirname(__FILE__) . DIRECTORY_SEPARATOR .'language'. DIRECTORY_SEPARATOR . 'english.php'));
        $this->iframe_div_style       = !$this->get_option('iframe_div_style') ? $this->_default_iframe_style : $this->get_option('iframe_div_style');
		if($this->debug){
			$this->log = new WC_Logger();
		}

        add_action('woocommerce_thankyou_txpmnts', array(&$this, 'payment_finished'));
        add_action('woocommerce_receipt_txpmnts', array(&$this, 'receipt_page'));
        add_action('woocommerce_update_options_payment_gateways_'.$this->id,
                array($this, 'process_admin_options'));
        add_action('woocommerce_api_wc_txpmnts', array($this, 'notification_handler'));

        add_action('before_woocommerce_pay', function()
        {
            $order_id           = absint( WC()->session->get( 'order_awaiting_payment' ) );
            $this->generate_iframe_html($order_id);
        });

		//load database and api classes
        //if(!class_exists('GPNDataAPI') || !class_exists('Database')){
            $this->autoLoad();
        //}

		if(is_admin())
		{
            add_thickbox();
            wp_localize_script( 'frontend-ajax', 'frontendajax', array( 'ajaxurl' => admin_url( 'admin-ajax.php' )));
            add_action( 'edit_form_advanced', function( $post )
            {
                //SSL must be enabled on server to use full api
                if (is_ssl())
                {
                    $order = new WC_Order(intval($post->ID));
                    $order_id = trim(str_replace('#', '', $order->get_order_number()));
                    global $wpdb;
                    $this->_db = new Database($wpdb->prefix, $wpdb->dbhost, $wpdb->dbname, $wpdb->dbuser, $wpdb->dbpassword);
                    $this->gpndata_order = $this->_db->getTransactionByOrderId($order_id, true);

                    //construct the request xml
                    $last_transaction_state = $this->_API->checkTransactionStatus($this->gpndata_order['transaction']['merchant_trans_id'], $this->gpndata_order['transaction']['gate_trans_id']);
                    $state													= $last_transaction_state->transaction->state;
                    $description											= $last_transaction_state->transaction->description;
                    $this->gpndata_order['transaction_current_state']   	= $state;
                    $this->gpndata_order['transaction_current_state_description'] = $description;
                }


                include_once(dirname(__FILE__) . DIRECTORY_SEPARATOR . 'templates' . DIRECTORY_SEPARATOR . 'operations.php');
                unset($order);
            });
		}
    }

    protected static $instance = NULL;

    public static function get_instance() {

        // create an object
        NULL === self::$instance and self::$instance = new self;

        return self::$instance; // return the object
    }

    public function get_admin_order(){
        if(is_admin())
            return $this->gpndata_order;

        return null;
    }

    //woocomm override, refunds are supported in different way
    public function process_refund(){
        return false;
    }

	protected function autoLoad(){

		$path = dirname(__FILE__);
		include_once($path . DIRECTORY_SEPARATOR . 'class' . DIRECTORY_SEPARATOR . 'gpndata.php');
        include_once($path . DIRECTORY_SEPARATOR . 'class' . DIRECTORY_SEPARATOR . 'database.php');
		global $wpdb;
		$this->_db = new Database($wpdb->prefix, $wpdb->dbhost, $wpdb->dbname, $wpdb->dbuser, $wpdb->dbpassword);
		$this->_API = new GPNDataAPI($this->apiuser, $this->apipassword, $this->apikey);
		$this->_API->transactionUrl = $this->gateway_url;
	}

	/**
     * Logging method
     * @param  string $message
     */
	public function log($message){
		if($this->debug){
			if(empty($this->log)){
				$this->log = new WC_Logger();
			}
			$this->log->add('TxPmnts', $message);
		}
	}

    public function receipt_page(){
        return true;
    }
	/**
     * Initialise Gateway Settings Form Fields
     */
	public function init_form_fields(){
        $this->form_fields = include (dirname(__FILE__) . DIRECTORY_SEPARATOR .'etc'. DIRECTORY_SEPARATOR . 'admin-options.php');
	}

	/**
     * Admin Panel Options
     */
	public function admin_options(){
        include (dirname(__FILE__) . DIRECTORY_SEPARATOR .'templates'. DIRECTORY_SEPARATOR . 'admin-options.php');
	}

	public function payment_fields(){
		if($this->description){
			echo wpautop(wptexturize($this->description));
		}
	}

	/**
     * Process the payment and return the result
     **/
	public function process_payment($order_id){
		$order = new WC_Order($order_id);
		return array(
				'result'   => 'success',
				'redirect' => add_query_arg('order', $order->id, add_query_arg('key', $order->order_key,
				                                                               get_permalink(wc_get_page_id('checkout'))))
		);
	}

    function generate_iframe_html($order_id)
    {
        $pay_for_order_id = get_query_var('order-pay');
        $order = $pay_for_order_id ?  new WC_Order($pay_for_order_id) : new WC_Order($order_id);
        //prevent user from paying for order once more, but give him an option to pay failed order
        // $order_status = $order->get_status();
        //&& $order_status !='failed' && $order_status !='cancelled'
        //TO DO: Handle this better, let the customer make the payment!
        if($pay_for_order_id ){
            $transaction =   $this->_db->getTransactionByOrderId($pay_for_order_id, true, true);
            $message = $this->set_order_status($transaction['transaction_adjustments'], $pay_for_order_id, false);
            echo '<h4>'.__('This order cannot be completed or modified. Please place new order.').'</h4>';
            echo '<p>'.__($message).'</p>';
            die();
        }
        if($order->get_items()){
            foreach($order->get_items() as $item){
				$products_list[] =  $item['qty'].' x '.$item['name'];
            }
        }

        do_action('woocommerce_checkout_before_order_review');
        $trans_id    = $order->id.'-'.time().rand(100,999); //Order ID must be unique, but we will need proper one later
        $return_base = $this->get_return_url($order);
        $return_url  = $return_base.'&txresult=[#STATUS#]';
        $description = implode(", ", $products_list);
        $order_description = empty($this->iframe_order_description) ? "Order # " . $description : $this->iframe_order_description. ' ' . $order_id;
        $country_iso = $this->iso2toiso3($order->billing_country);
        $state = $country_iso == 'USA' ? $order->get_billing_state() : '';
        $iframe_div_style = isset($this->iframe_div_style) && !empty($this->iframe_div_style) ? $this->iframe_div_style : $this->_default_iframe_style;

        echo  '<style>'.$iframe_div_style.'</style>'. '<p>'.$this->description .'</p>
            <div id="outer" class="order_details">
                    <div id="inner" class="gpn-payform"
                                    data-locale="'.$this->iframe_language.'"
                                    data-merchant-id="'.$this->pretty_name.'"
                                    data-amount="'.sprintf('%.2F', $order->order_total).'"
                                    data-success-url="'.$return_url.'"
                                    data-failure-url="'.$return_url.'"
                                    data-currency="'.get_woocommerce_currency().'"
                                    data-company="'.$this->merchantname.'"
                                    data-description="'.$order_description.'"
                                    data-logo="'.$this->logo.'"
                                    data-skip-customerinfo="true"
                                    data-auto-open="true"
                                    data-order-id="'.$trans_id.'"
                                    data-customer-id="'.$order->billing_email.'"
                                    data-customer-email="'.$order->billing_email.'"
                                    data-customer-phone="'.preg_replace('/\D/', '', $order->billing_phone).'"
                                    data-customer-firstname="'.$order->billing_first_name.'"
                                    data-customer-lastname="'.$order->billing_last_name.'"
                                    data-customer-addressline="'.$order->billing_address_1.'"
                                    data-customer-zipcode="'.$order->billing_postcode.'"
                                    data-customer-city="'.$order->billing_city.'"
                                    data-customer-state="' . $state . '"
                                    data-customer-country="'.$country_iso.'">
                                </div>
            </div>
            <script src="'.$this->iframe_url.'"></script>';
    }

	public function payment_finished($order_id){

        $message = $this->set_order_status($_POST['status'], $order_id);
        echo '<section class="woocommerce-order-details">
	            <h2 class="woocommerce-order-details__title">Payment details</h2>
                    <table class="woocommerce-table woocommerce-table--order-details shop_table order_details">
                        <th class="woocommerce-table__product-name product-name">'. $message .'</th>
                    </table>
              </section>';

        if(isset($message)){
            return false;
        }

		if($_GET['txresult'] != 'SUCCESS'){
			echo '<p style="color: red; font-weight: bold;">'.
            __('We are sorry, but your payment for this order via the Payment Gateway was unsuccessful. The order has now been cancelled. Please re-issue your order.',
               'woo-txpmnts').'</p>';
			//$order->cancel_order(__('Payment failed', 'woo-txpmnts'));

			return false;
		}

		echo '<p>'.__('Your payment has been processed successfully! Thank you for shopping with us.').'</p>';

		return true;
	}

    public function set_order_status($status, $order_id, $update_status = true, $transaction = null){
        $order = new WC_Order($order_id);
        $message = null;
		if(!$order){
            $message = 'Error: Invalid order ID.';
			return $message;
		}
        if($transaction){
            switch ($status){
                case TransactionStatus::PENDING:
                    if($transaction['notification_type'] != NotificationResponses::RefundNotification)
                        $this->_db->updateTransactionByMerchantTransactionId(array('merchant_trans_id' => $transaction['transaction']['merchant_trans_id'], 'gate_trans_id' => $transaction['transaction']['gate_trans_id']));
                    $order->update_status('pending', sprintf( __( 'GPN DATA - order number %s status PENDING.', 'woo-txpmnts' ), strval($order_id)));
                    break;
                case TransactionStatus::DECLINED:
                    if($transaction['notification_type'] == NotificationResponses::RefundNotification)
                        goto LBL_REFUND;

                    $this->_db->updateTransactionByMerchantTransactionId(
                        array('merchant_trans_id' => $transaction['transaction']['merchant_trans_id'],
                              'gate_trans_id' => $transaction['transaction']['gate_trans_id'],
                              'is_declined' => 1,
                              'is_refund_pending' => 0,
                              'is_captured' => 0));
                    $order->add_order_note( sprintf( __( 'Order number %s has been DECLINED', 'woo-txpmnts' ), strval($order_id)) );
                    break;
                case TransactionStatus::AUTHORIZED:
                    $this->_db->updateTransactionByMerchantTransactionId(array('merchant_trans_id' => $transaction['transaction']['merchant_trans_id'], 'gate_trans_id' => $transaction['transaction']['gate_trans_id'], 'is_authorized' => 1));
                    $order->add_order_note( sprintf( __( 'Order number %s has been AUTHORIZED', 'woo-txpmnts' ), strval($order_id)) );
                    break;
                case TransactionStatus::CAPTURED:
                    $this->_db->updateTransactionByMerchantTransactionId(array('merchant_trans_id' => $transaction['transaction']['merchant_trans_id'], 'is_authorized' => 0, 'gate_trans_id' => $transaction['transaction']['gate_trans_id'], 'is_captured' => 1));
                    if($transaction['inserted'] || $transaction['transaction']['is_authorized'] || $transaction['transaction']['is_captured'] == 0){
                        $order->update_status('completed', sprintf( __( 'Order number %s has been CAPTURED', 'woo-txpmnts' ), strval($order_id)));
                    }
                    break;
                case TransactionStatus::CANCELED:
                    $this->_db->updateTransactionByMerchantTransactionId(array('merchant_trans_id'=> $transaction['merchant_trans_id'], 'is_voided' => 1));
                    $order->update_status('cancelled', sprintf( __( 'GPN DATA - order number %s status CANCELED.', 'woo-txpmnts' ), strval($order_id)));
                    break;
                default:
                    LBL_REFUND:
                    if($status == TransactionStatus::REFUNDED || (isset($transaction['notification_type'])
                        && $transaction['notification_type'] == NotificationResponses::RefundNotification
                        && ($status == TransactionStatus::SUCCESS || $status == TransactionStatus::DECLINED))){
                        if ($transaction['transaction']['is_refund_pending']){
                            $this->_db->updateTransactionByMerchantTransactionId(array('merchant_trans_id'=> $transaction['transaction']['merchant_trans_id'], 'is_refund_pending' => 0));
                            $description = $status == TransactionStatus::DECLINED ? 'Refund declined' : 'Refunded amount: ' . $transaction['transaction_adjustment']['amount'];
                            $transaction['transaction_adjustment']['status'] = $status == TransactionStatus::DECLINED ? TransactionStatus::DECLINED : TransactionStatus::REFUNDED;
                            $transaction['transaction_adjustment']['state'] = $status == TransactionStatus::DECLINED ? TransactionStatus::DECLINED : TransactionStatus::REFUNDED;
                            if(!$transaction['transaction_adjustment']['description'])
                                $transaction['transaction_adjustment']['description'] = $description;
                            $this->_db->addTransactionAdjustmentFromArray($transaction['transaction_adjustment']);
                            //refund status should be set only in case of full refund, leave it like it is for now TO DO!
                            $order->update_status('refunded', $description);
                        }
                    }
                    break;
            }
        }
        else{
            switch ($status){
                case TransactionStatus::AUTHORIZED:
                    $message = 'This order is pending completion and you will be notified by email once order has been completed.';
                    if($update_status)
                        $order->update_status('pending', sprintf( __( 'GPN DATA - order number %s status AUTHORIZED', 'woo-txpmnts' ), strval($order_id)));
                    break;
                case TransactionStatus::SUCCESS:
                    $message = 'Your payment has been processed successfully! Thank you for shopping with us.';
                    break;
                case TransactionStatus::CAPTURED:
                    $message = 'Your payment has been processed successfully! Thank you for shopping with us.';
                    if($update_status)
                        $order->add_order_note( sprintf( __( 'GPN DATA - order number %s status CAPTURED', 'woo-txpmnts' ), strval($order_id)) );
                    break;
                case TransactionStatus::DECLINED:
                    $message = 'Unfortunately your payment has been declined. Please check your credit card details and try again.';
                    if($update_status)
                        $order->update_status('failed', sprintf( __( 'GPN DATA - Customer payment for order number %s has been declined.', 'woo-txpmnts' ), strval($order_id)));
                    break;
                default:
                    break;
            }
        }
        unset($order);
        return $message;
    }

	public function notification_handler(){

		try{
			if(empty($_POST['strdata'])){
				throw new Exception('Missing strdata parameter', -1);
			}

			// There is some weird concept in wordpress, to always execute wp_magic_quotes().
			// So apparently WP is always adding slashes. WTF?
			$_POST['strdata'] = stripslashes($_POST['strdata']);

			$xml_data = simplexml_load_string($_POST['strdata']);
			if(empty($xml_data)){
				throw new Exception('Invalid strdata parameter value', -2);
			}

			if(!($xml_data->apiUser == $this->apiuser && $xml_data->apiPassword == $this->apipassword)){
				throw new Exception('Invalid api credentials', -3);
			}

			if(!class_exists('GPNDataAPI')){
				$this->autoLoad();
			}
            $type = intval($xml_data->apiCmd);
			if ($this->_API->validateNotificationChecksum($xml_data)){

				switch ($type) {
					case NotificationResponses::TransactionStateNotification://This is Transaction Notification

						$message = $this->processTransactionNotification($xml_data, $type);
						break;
					case NotificationResponses::RefundNotification:

                        $message = $this->processTransactionNotification($xml_data, $type);//This is Refund Notification
						break;
					case NotificationResponses::DisputeNotification: //This is Chargeback Notification

                        $message = $this->processTransactionNotification($xml_data, $type);
						break;
					default:
						unset($xml_data);
				}
			}
            ob_clean();
			$resstr = '
			<?xml version="1.0" encoding="utf-8" ?>
			<transaction>
				<result>OK</result>
				<plugin-message>'.$message.'</plugin-message>
				<gatetransid>'.$xml_data->gatetransid.'</gatetransid>
			</transaction>';
			echo $resstr;
			unset($xml_data);

		}
		catch(Exception $e){
            ob_clean();
			echo('<?xml version="1.0" encoding="utf-8" ?>
					<response>
						<result>ERROR</result>
						<gatetransid></gatetransid>
						<merchanttransid></merchanttransid>
						<errorcode>'.$e->getCode().'</errorcode>
						<errormessage>'.$e->getMessage().'</errormessage>
					</response>');
        }

		// Below is required to display anything, because output is buffered at this point
		// by WC_API::handle_api_requests().
		die(' ');
	}

	public function processTransactionNotification($xml_data, $type){

		$merchanttransid            = strval($xml_data->merchanttransid);
		$transaction_gatetransid    = strval($xml_data->gatetransid);
		$amount                     = strval($xml_data->amount);
		$curcode                    = strval($xml_data->curcode);
		$status                     = strval($xml_data->status);
		$state                      = strval($xml_data->state);
		$description                = strval($xml_data->description);
        $reference                  = strval($xml_data->transref);
		$message					= 'OK';
        $installment_count = isset($xml_data->installment) ? intval($xml_data->installment) : 0;
        list($orderId, ) = explode('-', $merchanttransid);
        $order = new WC_Order($orderId);
		if(!$order){
			return 'Invalid WooCommerce order ID';
		}

        $order_id					= $order->id;
        if($type == NotificationResponses::DisputeNotification){
            $chgbk_chbDate = strval($xml_data->disputeDate);
            $chgbk_reasonCode = strval($xml_data->reasonCode);
            $text_transaction_status_by_notification = strval($xml_data->reasonText);
            $text_transaction_status_by_notification.'. Reason code: '.$chgbk_reasonCode.'\n ARN:'.strval($xml_data->arn).'\n Dispute date: '.$chgbk_chbDate;
        }

		$text_transaction_status_by_notification = $description;//$this->language->get('text_transaction_status_by_notification');
        $transaction_data			= array(
                                            'gate_trans_id'		=> "$transaction_gatetransid",
                                            'status'			=> "$status",
                                            'state'				=> "$state",
                                            'description'		=> "$description",
                                            'amount'			=> "$amount",
                                            'additional_text'	=> "$text_transaction_status_by_notification"
                                            );
        $transaction = array
               (
                   'order_id' => $order_id,
                   'transaction_ref' => $reference,
                   'gate_trans_id' => $transaction_gatetransid,
                   'merchant_trans_id' => $merchanttransid,
                   'curcode' =>  $curcode,
                   'payment_amount' => $amount,
                   'is_captured' => $status == TransactionStatus::CAPTURED ? 1 : 0,
                   'is_authorized' => $status == TransactionStatus::AUTHORIZED ? 1 : 0,
                   'is_declined' => $status == TransactionStatus::DECLINED ? 1 : 0,
               );

        if($installment_count > 0){
            $transaction['installment_count'] =  $installment_count;
        }

        $inserted = $this->_db->addTransactionIfDoesNotExist($transaction);

        if($inserted){
            $message = $message. ' Payment data was inserted to transactions table.';
        }
        else{
            $transaction = $this->_db->getTransactionByMerchantTransactionId($merchanttransid);
        }

        $refund_amount = null;
        if($type == NotificationResponses::RefundNotification){
            $last_status_exist =  strval($this->_db->getLastTransactionAdjustment($transaction_gatetransid)) == $status;
            if(!$last_status_exist)
                $refund_amount = (floatval($transaction['transaction']['payment_amount']) - floatval($amount));
        }

        $transaction['inserted'] = $inserted;
        $transaction['notification_type'] = $type;
        $transaction['transaction_adjustment'] = $transaction_data;
        $message = $message. $this->set_order_status($status, $order_id, true, $transaction);
        unset($transaction['adjustment']);
        unset($transaction['notification_type']);
        unset($transaction['inserted']);

        if($status == TransactionStatus::CHARGEBACK){
            $this->_db->addTransactionAdjustmentFromArray($transaction_data);
        }

        if(!$inserted){
            $t =  array(
                      'gate_trans_id' => $transaction_gatetransid,
                      'is_captured' => ((intval($transaction['transaction']['is_captured']) == 1 ? $transaction['transaction']['is_captured'] : ((intval($transaction['transaction']['is_captured']) == 0 && $status == TransactionStatus::CAPTURED) ? 1 : 0))),
                      'is_authorized' => (intval($transaction['transaction']['is_authorized']) == 1 || $status == TransactionStatus::AUTHORIZED) ? 1 : 0,
                      'is_voided' => (intval($transaction['transaction']['is_voided']) == 1 ? 1 : (intval($transaction['transaction']['is_voided']) == 0 && $status == TransactionStatus::CANCELED) ? 1 : 0),
                      'is_refunded' => (intval($transaction['transaction']['is_refunded']) == 1 ? 1 : (intval($transaction['transaction']['is_refunded']) == 0 && $status == TransactionStatus::REFUNDED) ? 1 : 0),
                      'is_refund_pending' => ($status == TransactionStatus::REFUND_PENDING) ? 1 : 0,
                      'is_chargeback' => ($type == 880 ? 1 : 0),
                      'payment_amount' => empty($refund_amount) ? $transaction['transaction']['payment_amount'] : $refund_amount
                  );

            if($installment_count > 0){
                $t['installment_count'] = $installment_count;
            }

            $update_result =  $this->_db->updateTransaction($t);
            $message = $message . " Transactions table status update result: $update_result.";
        }

        $message = $message . " Table history order transaction row was successfully created with status $status!";

		return $message;
    }

    function iso2toiso3($iso2){
        return $this->_API->mapIso($iso2);
    }

    function perfromTransactionAction($is_full_transaction_amount, $transaction, $action) {
        try
        {
            $payment_amount = $transaction['transaction']['payment_amount'];
            $api_request_result = null;
            $adjustment_data = array(
                          'gate_trans_id' => $transaction['transaction']['gate_trans_id'],
                          'amount' => $transaction['amount']
                          );
            $transaction_data = array('gate_trans_id' => $transaction['transaction']['gate_trans_id']);
            switch ($action){
                case 'capture':
                    $api_request_result = $this->_API->requestCaptureAuthorization($transaction['transaction']['gate_trans_id'], $transaction['amount']);
                    $adjustment_data['additional_text'] = $is_full_transaction_amount ? 'Captured' : 'Payment was partitialy captured. Original transaction amount: '.$transaction['transaction']['payment_amount'].' Captured amount: '.$transaction['amount'];
                    $transaction_data['is_captured'] = 1;
                    $transaction_data['payment_amount'] = (floatval($transaction['transaction']['payment_amount']) - (floatval($transaction['transaction']['payment_amount']) - floatval($transaction['amount'])));
                    break;
                case 'refund':
                    $api_request_result = $this->_API->refundTransaction($transaction['transaction']['gate_trans_id'], $transaction['amount'], $transaction['reason']);
                    $adjustment_data['additional_text'] = $is_full_transaction_amount ? 'Refunded' : 'Payment was partitialy refunded. Partial refund amount: '.$transaction['amount'];
                    $transaction_data['payment_amount'] = (floatval($transaction['transaction']['payment_amount']) - floatval($transaction['amount']));
                    break;
                case 'void':
                    $api_request_result = $this->_API->requestCancelAuthorization($transaction['transaction']['gate_trans_id']);
                    $transaction_data['is_voided'] = 1;
                    $adjustment_data['amount'] = $transaction['transaction']['payment_amount'];
                    break;
                default:
            }

            $result = $api_request_result->getTransactionResult();
            //if it was error append it here and return transaction result
            if($result == TransactionStatus::ERROR){
                $transaction['error'] = 'Error occured while perfoming action: '. $api_request_result->getErrorMsg();
                return $transaction;
            }
            if($result == TransactionStatus::SUCCESS){
                $adjustment_data['description'] = strval($api_request_result->getResponseValue('description'));
                if($action == 'refund'){
                    $transaction_data['is_refunded'] = 1;
                    $transaction['transaction']['is_refund_pending'] = 1;
                    $transaction['transaction_adjustment'] = $adjustment_data;
                    $this->set_order_status(TransactionStatus::REFUNDED, $transaction['transaction']['order_id'],true, $transaction);
                    unset($adjustment_data);
                }
                if($action == 'capture'){
                    $this->set_order_status(TransactionStatus::CAPTURED, $transaction['transaction']['order_id'],true, $transaction);
                    $result = TransactionStatus::CAPTURED;
                }
                if($action == 'void'){
                    $this->set_order_status(TransactionStatus::CANCELED, $transaction['transaction']['order_id'],true, $transaction);
                    $result = TransactionStatus::CANCELED;

                }
            }
            if ($result == TransactionStatus::REFUND_PENDING || $result == TransactionStatus::REQUEST_PENDING ||
                $result == TransactionStatus::PENDING){
                $adjustment_data['description'] = strval($api_request_result->getResponseValue('description'));
                $adjustment_data['additional_text'] = 'Pending on action: '.strtoupper($transaction['action']). ' for amount: '.$transaction['amount'];
                $transaction_data['is_refund_pending'] = ($result == TransactionStatus::REQUEST_PENDING && $transaction['action'] == 'refund' ? 1: 0);
                $transaction_data['payment_amount'] = $payment_amount;
            }

            if($adjustment_data){
                $adjustment_data['status'] = $result;
                $adjustment_data['state'] = $result;
                $this->_db->addTransactionAdjustmentFromArray($adjustment_data);
            }

            $this->_db->updateTransaction($transaction_data);

            return $this->_db->getTransactionByOrderId($transaction['transaction']['order_id'], true);
        }
        catch (Exception $exception)
        {
            $transaction['error'] = $exception->getMessage();
        }
        return $transaction;
    }
}

/**
 * Add the gateway to WooCommerce
 **/
function add_txpmnts_gateway($methods){
	$methods[] = 'WC_txpmnts';
	return $methods;
}

add_filter('woocommerce_payment_gateways', 'add_txpmnts_gateway');
if (is_admin()){
    add_action('wp_ajax_payment_functions', 'payment_functions');

    function payment_functions() {
        $result = array();
        if ( !wp_verify_nonce( $_POST['data_nonce'], "payment_functions_nonce")) {
            $result['status'] = 'error';
            $result['message'] = 'No naughty business please';
        }
        if(empty($_POST['order_id'])){
            $result['status'] = 'error';
            $result['message'] = 'Order is not loaded';
        }
        $gateway = WC_txpmnts::get_instance();
        $transaction = $gateway->_db->getTransactionByOrderId($_POST['order_id']);
        $transaction_amount = sprintf('%.2F',str_replace(',','', $transaction['transaction']['payment_amount']));
        $requested_amount   =  sprintf('%.2F', str_replace(',','', $_POST['payment_amount']));
        $transaction['amount'] = $requested_amount;
        $transaction['action'] = $_POST['payment_action'];
        $transaction['reason'] = $_POST['reason'];

        $is_full_transaction_amount = $transaction_amount == $requested_amount;
        $transaction = $gateway->perfromTransactionAction($is_full_transaction_amount, $transaction,  $transaction['action']);
        if($transaction['error']){
            $result['status'] = 'error';
            $result['message'] = $transaction['error'];
        }
        echo json_encode($result);
		wp_die();
	}
}
register_activation_hook( __FILE__, 'txpmnts_activate' );
function txpmnts_activate() {
    include_once(__DIR__ . DIRECTORY_SEPARATOR . 'class' . DIRECTORY_SEPARATOR . 'database.php');
    global $wpdb;
    $db = new Database($wpdb->prefix, $wpdb->dbhost, $wpdb->dbname, $wpdb->dbuser, $wpdb->dbpassword);
    $db->createTables();
}

register_deactivation_hook( __FILE__, 'txpmnts_deactivation' );
function txpmnts_deactivation() {
    include_once(__DIR__ . DIRECTORY_SEPARATOR . 'class' . DIRECTORY_SEPARATOR . 'database.php');
    global $wpdb;
    $db = new Database($wpdb->prefix, $wpdb->dbhost, $wpdb->dbname, $wpdb->dbuser, $wpdb->dbpassword);
    $db->dropTables();
}

/**
 * Handles user cancel of order on GW.
 **/
add_action( 'woocommerce_order_status_cancelled', 'manage_user_order_cancel');

function manage_user_order_cancel($order_id){
    $order = new WC_Order($order_id);
    if($order){
        $gateway = WC_txpmnts::get_instance();
        if($gateway->_db){
            $transaction = $gateway->_db->getTransactionByOrderId($order_id);
            if($transaction['transaction']['is_authorized']){
                $result = $gateway->perfromTransactionAction(false, $transaction, 'void');
                if($result['error']){
                    echo "<script>alert('There was an error while trying to cancel payment on payment gateway. Please contact the merchant!')</script>";
                }
                if ($result['transaction']['is_voided']){
                    echo "<script>alert('Order and payment were cancelled!')</script>";
                }
            }
            if($transaction['transaction']['is_declined']){
                echo "<script>alert('Payment for this order was declined. Order cannot be cancelled.')</script>";
            }
            $order->add_order_note( sprintf( __( 'User canceled or try to cancel the order %s', 'woo-txpmnts' ), strval($order_id)) );
        }
    }
    unset($order);
    if ( is_user_logged_in())
        wp_redirect( get_permalink( get_option('woocommerce_myaccount_page_id') ) );
}

add_filter( 'woocommerce_thankyou_order_received_text', function($thank_you_msg){
    $status = $_POST['status'];
    if ($status == TransactionStatus::ERROR){
        $thank_you_msg = 'There was an error processing your payment';
    }
    if ($status == TransactionStatus::DECLINED){
        $thank_you_msg = 'Unfortunately your payment has been declined. Please check your credit card details and try again. ';
    }

    return $thank_you_msg;
});

add_filter( 'woocommerce_checkout_fields' , 'custom_override_checkout_fields' );

// Our hooked in function - $fields is passed via the filter!
function custom_override_checkout_fields( $fields ) {
    if(isset($fields['billing']['billing_address_2'])){
        unset($fields['billing']['billing_address_2']);
    }
    if(isset($fields['billing']['billing_state'])){
            $fields['billing']['billing_state']['value'] = WC()->checkout->get_value('billing_country');
        $fields['billing']['billing_state']['required'] = false;
    }

    return $fields;
}